@@ -45,6 +45,8 @@
 #include "fv_View.h"
 #include "pt_PieceTable.h"
 #include "ap_Win32Toolbar_FontCombo.h"
+#include "ap_Win32App.h"
+
 
 #ifndef TBSTYLE_EX_DRAWDDARROWS
 #define TBSTYLE_EX_DRAWDDARROWS 0x00000001
@@ -99,7 +101,7 @@ foo_Bitmap_container::~foo_Bitmap_container()
 	const UT_uint32 nItems = m_bitmaps_vector.getItemCount();
 	for (UT_uint32 iItem = nItems; iItem; --iItem)
 	{
-		void* p = m_bitmaps_vector.getNthItem(iItem - 1);
+		void* p = (void*)m_bitmaps_vector.getNthItem(iItem - 1);
 		// This is really, *REALLY* bad! The same entity that created this
 		// bitmap should also be responsible to destroy it. Time permitting,
 		// refactor all this code to make it correct.
@@ -131,6 +133,10 @@ EV_Win32Toolbar::~EV_Win32Toolbar(void)
 
 	if (m_pFontCtrl)
 		delete m_pFontCtrl;
+
+	for (UT_uint32 c=0; c < m_vecOrgStylesNames.getItemCount(); c++)	
+		delete m_vecOrgStylesNames.getNthItem(c);
+
 }
 
 bool EV_Win32Toolbar::toolbarEvent(XAP_Toolbar_Id id,
@@ -259,7 +265,12 @@ LRESULT CALLBACK EV_Win32Toolbar::_ComboWndProc( HWND hWnd, UINT uMessage, WPARA
 				break;
 			}
 								
-			DRAWITEMSTRUCT* dis = (DRAWITEMSTRUCT*)lParam;			
+			DRAWITEMSTRUCT* dis = (DRAWITEMSTRUCT*)lParam;
+			if((UT_sint32)dis->itemData < 0)
+			{
+				return TRUE;
+			}
+			
 			XAP_Toolbar_ControlFactory * pFactory = t->m_pWin32App->getControlFactory();			
 
 			if (!t->m_pFontCtrl)
@@ -273,7 +284,7 @@ LRESULT CALLBACK EV_Win32Toolbar::_ComboWndProc( HWND hWnd, UINT uMessage, WPARA
 			LOGFONT logfont;
 			SIZE size;
 			HFONT hfontSave;
-			const UT_Vector * v = t->m_pFontCtrl->getContents();
+			const UT_GenericVector<const char*> * v = t->m_pFontCtrl->getContents();
 			const UT_Vector * vcharSet = t->m_pFontCtrl->getFontsCharset();						
 			const char * sz  = (const char *)v->getNthItem(dis->itemData);			
 			const int nCharset = (int)vcharSet->getNthItem(dis->itemData);						
@@ -364,34 +375,26 @@ LRESULT CALLBACK EV_Win32Toolbar::_ComboWndProc( HWND hWnd, UINT uMessage, WPARA
 			{								
 				case CBN_SELCHANGE:
 				{
-					// are we currently dropped down?
-					if (!SendMessage(hWnd, CB_GETDROPPEDSTATE, 0, 0))
-						break;
-
-					// yep, we're done 
-					SendMessage(hWnd, CB_SHOWDROPDOWN, (WPARAM) FALSE, 0);
-
-					UT_sint32 iSelected = SendMessage(hWnd, CB_GETCURSEL, 0, 0);		
+					UT_sint32 iSelected = SendMessage(hWnd, CB_GETCURSEL, 0, 0);
 									
 					if(iSelected != -1)
 					{							
-						EV_Win32Toolbar * t = (EV_Win32Toolbar *) GetWindowLong(hWnd, GWL_USERDATA);
-						UT_ASSERT(t);
-
-						
-						UINT u = GetDlgCtrlID(hWnd);
-						XAP_Toolbar_Id id = t->ItemIdFromWmCommand(u);
-
 						static UT_UCSChar ucs_buf[COMBO_BUF_LEN];
 						char buf[COMBO_BUF_LEN];
 
-						UT_uint32 dataLength = GetWindowText(hWnd, buf, COMBO_BUF_LEN);
+						UT_uint32 dataLength = SendMessage(hWnd, CB_GETLBTEXT, iSelected, (LPARAM)&buf);
 
 						UT_UCS4_strcpy_char(ucs_buf, buf);
 						UT_UCSChar * pData = (UT_UCSChar *) ucs_buf;	// HACK: should be void *
-						
+
+						EV_Win32Toolbar * t = (EV_Win32Toolbar *) GetWindowLong(hWnd, GWL_USERDATA);
+						UT_ASSERT(t);
+
 						if(dataLength)
 						{	
+							UINT u = GetDlgCtrlID(hWnd);
+							XAP_Toolbar_Id id = t->ItemIdFromWmCommand(u);
+
 							// If is a STYLE_NAME we should pass the internal one, not the localised				
 							if (id==AP_TOOLBAR_ID_FMT_STYLE)
 							{									
@@ -403,12 +406,13 @@ LRESULT CALLBACK EV_Win32Toolbar::_ComboWndProc( HWND hWnd, UINT uMessage, WPARA
 								// Find the proper non-localised text                                                                             					
 								XAP_Toolbar_ControlFactory * pFactory = t->m_pWin32App->getControlFactory();
 								EV_Toolbar_Control * pControl = pFactory->getControl(t, AP_TOOLBAR_ID_FMT_STYLE);                                                                         
-								const UT_Vector * v = pControl->getContents();                                                                          
+								const UT_GenericVector<const char*> * v = pControl->getContents();                                                                          
 								AP_Win32Toolbar_StyleCombo * pStyleC = static_cast<AP_Win32Toolbar_StyleCombo *>(pControl);
 								pStyleC->repopulate();                                                                                                
 
-								nData  = SendMessage(hWnd, CB_GETITEMDATA, iSelected, 0);                                                         													
-								UT_UCS4_strcpy_char(ucs_buf, (char *)v->getNthItem(nData));				
+								nData  = SendMessage(hWnd, CB_GETITEMDATA, iSelected, 0);								
+
+								UT_UCS4_strcpy_char(ucs_buf, t->m_vecOrgStylesNames.getNthItem(nData)->utf8_str());								
 								DELETEP(pControl);
 							}
 							
@@ -646,8 +650,7 @@ bool EV_Win32Toolbar::synthesize(void)
 						
 						if (id==AP_TOOLBAR_ID_FMT_FONT)
 							dwStyle |= CBS_OWNERDRAWFIXED;
-							
-							
+
 						HWND hwndCombo = CreateWindowEx ( 0L,   // No extended styles.
 							"COMBOBOX",						// Class name.
 							"",								// Default text.
@@ -668,7 +671,7 @@ bool EV_Win32Toolbar::synthesize(void)
 						{
 							pControl->populate();
 
-							const UT_Vector * v = pControl->getContents();
+							const UT_GenericVector<const char*> * v = pControl->getContents();
 							UT_ASSERT(v);
 
 							SendMessage(hwndCombo, WM_SETREDRAW, FALSE,0);
@@ -1030,10 +1033,10 @@ bool EV_Win32Toolbar::_refreshItem(AV_View * pView, const EV_Toolbar_Action * pA
 			{
 				bool bGrayed = EV_TIS_ShouldBeGray(tis);
 				bool bString = EV_TIS_ShouldUseString(tis);
-				
 				HWND hwndCombo = _getControlWindow(id);
 				UT_return_val_if_fail(hwndCombo, true);
-
+				UT_UTF8String utf8;
+				UT_String str;
 				
 				// NOTE: we always update the control even if !szState
 				if (!szState)
@@ -1045,48 +1048,26 @@ bool EV_Win32Toolbar::_refreshItem(AV_View * pView, const EV_Toolbar_Action * pA
 				}					
 				
 				
-				// are we currently dropped down?
-			
 				// Find the proper non-localised text
-		
-				EV_Win32Toolbar * t = (EV_Win32Toolbar *) GetWindowLong(hwndCombo, GWL_USERDATA);
-				XAP_Toolbar_ControlFactory * pFactory = t->m_pWin32App->getControlFactory();			
-
-				EV_Toolbar_Control * pControl = pFactory->getControl(t, AP_TOOLBAR_ID_FMT_STYLE);			
-				const UT_Vector * v = pControl->getContents();				
+				const char* pLocalised = szState;
+				if (id==AP_TOOLBAR_ID_FMT_STYLE)
+				{
+						pt_PieceTable::s_getLocalisedStyleName(pLocalised, utf8);
+						pLocalised = utf8.utf8_str();
+						str = AP_Win32App::s_fromUTF8ToAnsi(pLocalised);
+						pLocalised = str.c_str();						
+				}
 					
-				AP_Win32Toolbar_StyleCombo * pStyleC = static_cast<AP_Win32Toolbar_StyleCombo *>(pControl);
-				pStyleC->repopulate();	
-				
-				//
-				// Is this a valid text?
-				//
-				UT_uint32 items = v->getItemCount();
-				bool bFound = false;
-				UT_uint32 k=0;
-				
-				for (k=0; k < items; k++)
-				{
-					if (strcmp((char *)v->getNthItem(k), szState)==0)
-					{
-						bFound = true;
-						break;
-					}	
-				}				
-				
-				char* pLocalised = (char *)szState;
-				
-				if (bFound)
-				{
-				 	pLocalised = (char *)v->getNthItem(k);					
-					pLocalised = (char *) pt_PieceTable::s_getLocalisedStyleName(pLocalised);
-				}								
 												
 				int idx = SendMessage(hwndCombo, CB_SELECTSTRING, (WPARAM)-1, (LPARAM)pLocalised);
 				if (idx==CB_ERR)
-					SetWindowText(hwndCombo, pLocalised);							
+				{
+                    // String not found, if the combo box has an "Other..." string, select it
+					const XAP_StringSet * pSS = XAP_App::getApp()->getStringSet();
+					pSS->getValue(XAP_STRING_ID_TB_Zoom_Percent);
+					idx = SendMessage(hwndCombo, CB_SELECTSTRING, (WPARAM)-1, (LPARAM)pSS->getValue(XAP_STRING_ID_TB_Zoom_Percent));
+				}
 	
-				DELETEP(pControl);	
 				//UT_DEBUGMSG(("refreshToolbar: ComboBox [%s] is %s and %s\n",
 				//			 m_pToolbarLabelSet->getLabel(id)->getToolbarLabel(),
 				//			 ((bGrayed) ? "disabled" : "enabled"),
@@ -1262,7 +1243,7 @@ bool EV_Win32Toolbar::repopulateStyles(void)
 	{
 		pLayoutItem = m_pToolbarLayout->getLayoutItem(i);
 		id = pLayoutItem->getToolbarId();
-	//	wd = (_wd *) m_vecToolbarWidgets.getNthItem(i);
+	//	wd = (_wd *) getNthItem(i);
 		if(id == AP_TOOLBAR_ID_FMT_STYLE)
 			break;
 	}
@@ -1288,7 +1269,7 @@ bool EV_Win32Toolbar::repopulateStyles(void)
 //
 // Now the combo box has to be refilled from this
 //						
-	const UT_Vector * v = pControl->getContents();
+	const UT_GenericVector<const char*> * v = pControl->getContents();
 	UT_ASSERT(v);
 	
 //
@@ -1309,19 +1290,28 @@ bool EV_Win32Toolbar::repopulateStyles(void)
 //
 	UT_uint32 items = v->getItemCount();
 	int	nItem;													    
+	UT_UTF8String utf8;
+	UT_String str;
+
+	for (UT_uint32 c=0; c < m_vecOrgStylesNames.getItemCount(); c++)	
+		delete m_vecOrgStylesNames.getNthItem(c);
+
+	m_vecOrgStylesNames.clear();
 	
-	for (UT_uint32 k=0; k < items; k++)
+	
+	for (UT_uint32 k=0; k < v->getItemCount(); k++)
 	{
-		char*	sz = (char *)v->getNthItem(k);
-		char*	pLocalised = sz;
+		const char*	sz = (char *)v->getNthItem(k);
+		const char*	pLocalised = sz;
 
-		pLocalised = (char *) pt_PieceTable::s_getLocalisedStyleName(sz);
+		pt_PieceTable::s_getLocalisedStyleName(sz, utf8);
+		pLocalised = utf8.utf8_str();
+		str = AP_Win32App::s_fromUTF8ToAnsi(pLocalised);
+		pLocalised = str.c_str();
 		
-		if (pLocalised!=sz)
-			int n=1;			
-
 		nItem = SendMessage(hwndCombo, CB_ADDSTRING,(WPARAM)0, (LPARAM)pLocalised);
-		SendMessage(hwndCombo, CB_SETITEMDATA,(WPARAM)nItem, (LPARAM)k);
+		m_vecOrgStylesNames.addItem (new UT_UTF8String ((char *)v->getNthItem(k)));
+		SendMessage(hwndCombo, CB_SETITEMDATA,(WPARAM)nItem, (LPARAM)m_vecOrgStylesNames.getItemCount()-1);
 	}
 //
 // Don't need this anymore and we don't like memory leaks in abi
@@ -1390,6 +1380,8 @@ void	EV_Win32Toolbar::onDropArrow(UINT cmd)
 
 			if (pAction->getItemType() == EV_TBIT_ColorFore)
 				fimpl->m_sColorFore = strColor;
+
+			toolbarEvent(id, NULL, 0);
 		}
 
 		pDialogFactory->releaseDialog(pDialog);
