@@ -39,21 +39,24 @@
 #include "fp_PageSize.h"
 #include "ut_string_class.h"
 #include "ut_misc.h"
-#include <fribidi.h>
 
 class UT_ByteBuf;
 class UT_GrowBuf;
 class pt_PieceTable;
 class PP_AttrProp;
+class PP_Revision;
+class PP_RevisionAttr;
 class pf_Frag_Strux;
 class PX_ChangeRecord;
 class PD_Style;
+class PD_DocIterator;
 class XAP_App;
 class fd_Field;
 class po_Bookmark;
 class fl_AutoNum;
 class fl_BlockLayout;
 class fp_Run;
+class UT_UTF8String;
 
 #ifdef PT_TEST
 #include "ut_test.h"
@@ -64,7 +67,10 @@ enum
 	PD_SIGNAL_UPDATE_LAYOUT,
 	PD_SIGNAL_REFORMAT_LAYOUT,
 	PD_SIGNAL_DOCPROPS_CHANGED_REBUILD,
-	PD_SIGNAL_DOCPROPS_CHANGED_NO_REBUILD
+	PD_SIGNAL_DOCPROPS_CHANGED_NO_REBUILD,
+	PD_SIGNAL_REVISION_MODE_CHANGED,
+	PD_SIGNAL_DOCNAME_CHANGED,
+	PD_SIGNAL_DOCDIRTY_CHANGED
 };
 
 /////////////////////////////////////////////////////////
@@ -143,18 +149,20 @@ enum
 // the creator (product) of this document. AbiWord, KWord, etc...
 #define PD_META_KEY_GENERATOR         "abiword.generator"
 
-// a helper class for keeping track of revisions in the document
-class PD_Revision
+class PD_DocumentDiff
 {
   public:
-	PD_Revision(UT_uint32 iId, UT_UCS4Char * pDesc):m_iId(iId),m_pDescription(pDesc){};
-	~PD_Revision(){delete m_pDescription;}
-	UT_uint32         getId()const{return m_iId;}
-	UT_UCS4Char *     getDescription() const {return m_pDescription;}
+	PD_DocumentDiff(bool bDel, PT_DocPosition p1, PT_DocPosition p2, UT_uint32 len)
+		:m_bDeleted(bDel), m_pos1(p1), m_pos2(p2), m_len(len){};
 
-  private:
-	UT_uint32 m_iId;
-	UT_UCS4Char * m_pDescription;
+#ifdef DEBUG
+	void _dump() const;
+#endif
+	
+	bool           m_bDeleted;
+	PT_DocPosition m_pos1;
+	PT_DocPosition m_pos2;
+	UT_uint32      m_len;
 };
 
 
@@ -168,26 +176,31 @@ class ABI_EXPORT PD_Document : public AD_Document
 public:
 	PD_Document(XAP_App *pApp);
 
+	virtual AD_DOCUMENT_TYPE getType() const {return ADDOCUMENT_ABIWORD;}
+	
 	virtual UT_Error		readFromFile(const char * szFilename, int ieft, const char * impProps = NULL);
 	virtual UT_Error		importFile(const char * szFilename, int ieft, bool markClean = false, bool bImportStylesFirst = true,
 									   const char * impProps = NULL);
 	virtual UT_Error		importStyles(const char * szFilename, int ieft, bool bDocProps = false);
 
 	virtual UT_Error		newDocument(void);
-	virtual bool			isDirty(void) const;
 
+	UT_Error                createRawDocument(void);
+	void                    finishRawCreation(void);
+
+	virtual bool			isDirty(void) const;
+	virtual void            forceDirty();
+	
 	virtual bool			canDo(bool bUndo) const;
 	virtual UT_uint32		undoCount(bool bUndo) const;
 	virtual bool			undoCmd(UT_uint32 repeatCount);
 	virtual bool			redoCmd(UT_uint32 repeatCount);
-
-	UT_Error				saveAs(const char * szFilename, int ieft, const char * expProps = NULL);
-	UT_Error   				saveAs(const char * szFilename, int ieft, bool cpy, const char * expProps = NULL);
-	UT_Error				save(void);
+	bool                    isDoingTheDo(void) const;
 
 	void					beginUserAtomicGlob(void);
 	void					endUserAtomicGlob(void);
-
+	void                    setMarginChangeOnly(bool b);
+	bool                    isMarginChangeOnly(void) const;
 	bool					insertObject(PT_DocPosition dpos,
 										 PTObjectType pto,
 										 const XML_Char ** attributes,
@@ -231,6 +244,18 @@ public:
 										   const XML_Char ** properties,
 										   PTStruxType pts);
 
+	bool					changeStruxFmt(PTChangeFmt ptc,
+										   PT_DocPosition dpos1,
+										   PT_DocPosition dpos2,
+										   const XML_Char ** attributes,
+										   const XML_Char ** properties);
+
+
+	bool					changeStruxFmtNoUndo(PTChangeFmt ptc,
+										   PL_StruxDocHandle sdh,
+										   const XML_Char ** attributes,
+										   const XML_Char ** properties);
+
 	bool					changeStruxForLists(PL_StruxDocHandle sdh,
 												const char * pszParentID);
 
@@ -238,24 +263,37 @@ public:
 										  PT_DocPosition dpos,
 										  PP_AttrProp *p_AttrProp);
 
+	bool                    changeLastStruxFmtNoUndo(PT_DocPosition dpos, PTStruxType pts,
+													 const XML_Char ** attrs, const XML_Char ** props,
+													 bool bSkipEmbededSections);
+	
+	bool                    changeLastStruxFmtNoUndo(PT_DocPosition dpos, PTStruxType pts,
+													 const XML_Char ** attrs, const XML_Char * props,
+													 bool bSkipEmbededSections);
+	
 	// the append- and insertBeforeFrag methods are only available while importing
 	// the document.
 
 	bool					appendStrux(PTStruxType pts, const XML_Char ** attributes, pf_Frag_Strux ** ppfs_ret = 0);
 	bool					appendStruxFmt(pf_Frag_Strux * pfs, const XML_Char ** attributes);
+	bool                    appendLastStruxFmt(PTStruxType pts, const XML_Char ** attrs, const XML_Char ** props,
+											   bool bSkipEmbededSections);
+	bool                    appendLastStruxFmt(PTStruxType pts, const XML_Char ** attrs, const XML_Char * props,
+											   bool bSkipEmbededSections);
 	bool					appendFmt(const XML_Char ** attributes);
-	bool					appendFmt(const UT_Vector * pVecAttributes);
+	bool					appendFmt(const UT_GenericVector<XML_Char*> * pVecAttributes);
 	bool					appendSpan(const UT_UCSChar * p, UT_uint32 length);
 	bool					appendObject(PTObjectType pto, const XML_Char ** attributes);
 	bool					appendFmtMark(void);
 	bool					appendStyle(const XML_Char ** attributes);
-	
+	bool                    changeStruxFormatNoUpdate(PTChangeFmt ptc ,PL_StruxDocHandle sdh,const XML_Char ** attributes);	
 	bool					insertStruxBeforeFrag(pf_Frag * pF, PTStruxType pts,
 												  const XML_Char ** attributes, pf_Frag_Strux ** ppfs_ret = 0);
 	bool					insertSpanBeforeFrag(pf_Frag * pF, const UT_UCSChar * p, UT_uint32 length);
 	bool					insertObjectBeforeFrag(pf_Frag * pF, PTObjectType pto,
 												   const XML_Char ** attributes);
 	bool					insertFmtMarkBeforeFrag(pf_Frag * pF);
+	bool					insertFmtMarkBeforeFrag(pf_Frag * pF, const XML_Char ** attributes);
 
 	pf_Frag *               findFragOfType(pf_Frag::PFType iType, UT_sint32 iSubtype = -1,
 										   const pf_Frag * pfStart = NULL);
@@ -272,13 +310,25 @@ public:
 	bool					notifyListeners(const pf_Frag_Strux * pfs,
 											pf_Frag_Strux * pfsNew,
 											const PX_ChangeRecord * pcr) const;
+	void					deferNotifications(void);
+	void					processDeferredNotifications(void);
 
+	// the first two of these functions just retrieve the AP with the given index; the latter two
+	// return AP that represents state of things with current revision settings
 	bool					getAttrProp(PT_AttrPropIndex indexAP, const PP_AttrProp ** ppAP) const;
 	bool					getSpanAttrProp(PL_StruxDocHandle sdh, UT_uint32 offset, bool bLeftSide,
 											const PP_AttrProp ** ppAP) const;
+
+	bool                    getAttrProp(PT_AttrPropIndex apIndx, const PP_AttrProp ** ppAP, PP_RevisionAttr ** pRevisions,
+										bool bShowRevisions, UT_uint32 iRevisionId, bool &bHiddenRevision) const;
+
+	bool                    getSpanAttrProp(PL_StruxDocHandle sdh, UT_uint32 offset, bool bLeftSide,
+											const PP_AttrProp ** ppAP,
+											PP_RevisionAttr ** pRevisions,
+											bool bShowRevisions, UT_uint32 iRevisionId,
+											bool &bHiddenRevision) const;
+	
 	const UT_UCSChar *		getPointer(PT_BufIndex bi) const; /* see warning on this function */
-	bool					getSpanPtr(PL_StruxDocHandle sdh, UT_uint32 offset,
-									   const UT_UCSChar ** ppSpan, UT_uint32 * pLength) const;
 	bool					getBlockBuf(PL_StruxDocHandle sdh, UT_GrowBuf * pgb) const;
 
 	bool					getBounds(bool bEnd, PT_DocPosition & docPos) const;
@@ -323,11 +373,18 @@ public:
 	bool                    deleteStruxNoUpdate(PL_StruxDocHandle sdh);
 	bool                    insertStruxNoUpdateBefore(PL_StruxDocHandle sdh, PTStruxType pts,const XML_Char ** attributes );
 	bool                    isStruxBeforeThis(PL_StruxDocHandle sdh,  PTStruxType pts);
+
+	// the function below does exactly what the name says -- returns the AP index; in revisions mode
+	// you would need the AP at that index to have revision attribute exploded -- use one of the
+	// functions below to retrieve props and attrs correctly reflecting revisions settings
 	PT_AttrPropIndex        getAPIFromSDH(PL_StruxDocHandle sdh);
-    bool                    getAttributeFromSDH(PL_StruxDocHandle sdh, const char * szAttribute, const char ** pszValue);
-    bool                    getPropertyFromSDH(PL_StruxDocHandle sdh, const char * szProperty, const char ** pszValue);
+    bool                    getAttributeFromSDH(PL_StruxDocHandle sdh, bool bShowRevisions, UT_uint32 iRevisionLevel,
+												const char * szAttribute, const char ** pszValue);
+	
+    bool                    getPropertyFromSDH(PL_StruxDocHandle sdh, bool bShowRevisions, UT_uint32 iRevisionLevel,
+											   const char * szProperty, const char ** pszValue);
 	// styles
-	void                    getAllUsedStyles(UT_Vector * pVecStyles);
+	void                    getAllUsedStyles(UT_GenericVector<PD_Style*> * pVecStyles);
 	PL_StruxFmtHandle       getNthFmtHandle(PL_StruxDocHandle sdh, UT_uint32 n);
 	bool					getStyle(const char * szName, PD_Style ** ppStyle) const;
 	PD_Style *				getStyleFromSDH(PL_StruxDocHandle sdh);
@@ -347,8 +404,7 @@ public:
 	void                    updateAllLayoutsInDoc( PL_StruxDocHandle sdh);
 	void					clearIfAtFmtMark(PT_DocPosition dpos);
 
-	const char *			getFileName() { return m_szFilename; }
-	UT_uint32				getLastSavedAsType() { return m_lastSavedAsType; }
+	virtual UT_uint32		getLastSavedAsType() const { return m_lastSavedAsType; }
 	UT_uint32				getLastOpenedType() { return m_lastOpenedType; }
 	XAP_App *				getApp() { return m_pApp; }
 	bool					updateFields(void);
@@ -356,6 +412,8 @@ public:
 									 UT_uint32 offset,
                                      fd_Field * &pField);
 	po_Bookmark * 			getBookmark(PL_StruxDocHandle sdh, UT_uint32 offset);
+	pf_Frag *               findBookmark(const char * pName, bool bEnd = false, pf_Frag * pfStart = NULL);
+	
 	void					setDontChangeInsPoint(void);
 	void					allowChangeInsPoint(void);
 	bool					getAllowChangeInsPoint(void) const;
@@ -364,13 +422,27 @@ public:
 	bool                    isEndFootnoteAtPos(PT_DocPosition pos);
 	UT_sint32               getEmbeddedOffset(PL_StruxDocHandle sdh,PT_DocPosition posOff, PL_StruxDocHandle & sdhEmbedded);
 
+	// TOC functions
+	bool                    isTOCAtPos(PT_DocPosition pos);
+
+	// FRAME function
+	bool                    isFrameAtPos(PT_DocPosition pos);
+	bool                    isEndFrameAtPos(PT_DocPosition pos);
+
 // Table functions
 
+	bool                    isTableAtPos(PT_DocPosition pos);
+	bool                    isCellAtPos(PT_DocPosition pos);
 	PL_StruxDocHandle       getEndTableStruxFromTableSDH(PL_StruxDocHandle tableSDH);
 	PL_StruxDocHandle       getEndCellStruxFromCellSDH(PL_StruxDocHandle cellSDH);
 	PL_StruxDocHandle       getEndTableStruxFromTablePos(PT_DocPosition posTable);
-	bool                    getRowsColsFromTableSDH(PL_StruxDocHandle tableSDH, UT_sint32 * numRows, UT_sint32 * numCols);
-	PL_StruxDocHandle       getCellSDHFromRowCol(PL_StruxDocHandle tableSDH, UT_sint32 row, UT_sint32 col);
+	bool                    getRowsColsFromTableSDH(PL_StruxDocHandle tableSDH,
+													bool bShowRevisions, UT_uint32 iRevisionLevel,
+													UT_sint32 * numRows, UT_sint32 * numCols);
+	PL_StruxDocHandle       getCellSDHFromRowCol(PL_StruxDocHandle tableSDH,
+												 bool bShowRevisions, UT_uint32 iRevisionLevel,
+												 UT_sint32 row, 
+												 UT_sint32 col);
 	void                    miniDump(PL_StruxDocHandle sdh, UT_sint32 nstruxes);
 
 	// List Functions
@@ -378,7 +450,7 @@ public:
 	fl_AutoNum *			getNthList(UT_uint32 i) const;
 	bool					enumLists(UT_uint32 k, fl_AutoNum ** pAutoNum);
 	UT_uint32				getListsCount(void) const;
-	void					addList(const fl_AutoNum * pAutoNum);
+	void					addList(fl_AutoNum * pAutoNum);
 	bool					appendList(const XML_Char ** attributes);
 	bool					fixListHierarchy(void);
 	void					removeList(fl_AutoNum * pAutoNum,PL_StruxDocHandle sdh );
@@ -399,6 +471,7 @@ public:
 	bool                    isRedrawHappenning(void) const {return m_bRedrawHappenning;}
 
 	// PageSize functions
+	bool                    convertPercentToInches(const char * szPercent, UT_UTF8String & sInches);
 	fp_PageSize				m_docPageSize;
 	bool					setPageSizeFromFile(const XML_Char ** attributes);
 
@@ -412,15 +485,23 @@ public:
 	/////////////////////////////////////////////////////////////////////////////
 	// Functions for dealing with revisions
 	//
-	bool                    isMarkRevisions() const{ return m_bMarkRevisions;}
-	void                    toggleMarkRevisions(){m_bMarkRevisions = m_bMarkRevisions ? false : true;}
-	UT_uint32               getRevisionId() const{ return m_iRevisionID;}
-	void                    setRevisionId(UT_uint32 iId) {m_iRevisionID  = iId;}
-	bool                    addRevision(UT_uint32 iId, UT_UCS4Char * pDesc);
-	bool                    addRevision(UT_uint32 iId, const UT_UCS4Char * pDesc, UT_uint32 iLen);
-	UT_Vector &             getRevisions() {return m_vRevisions;}
-	UT_uint32               getHighestRevisionId() const;
-	const PD_Revision *     getHighestRevision() const;
+	virtual void            setMarkRevisions(bool bMark);
+	// primarly for use by the PieceTable
+	void                    setMarkRevisionsNoNotify(bool bMark) {AD_Document::setMarkRevisions(bMark);}
+	
+	virtual bool            acceptRejectRevision(bool bReject,
+												 UT_uint32 iStart,
+												 UT_uint32 iEnd,
+												 UT_uint32 iLevel);
+
+	virtual bool            rejectAllHigherRevisions(UT_uint32 iLevel);
+	virtual bool            acceptAllRevisions();
+
+	const PP_AttrProp *     explodeRevisions(PP_RevisionAttr *& pRevisions, const PP_AttrProp * pAP,
+											 bool bShow, UT_uint32 iId, bool &bHiddenRevision) const;
+	
+	virtual void            purgeRevisionTable();
+
 	void					notifyPieceTableChangeStart(void);
 	void					notifyPieceTableChangeEnd(void);
 
@@ -439,15 +520,10 @@ public:
 	inline bool areStylesLocked () const { return m_bLockedStyles; }    // See also lockStyles
 	void lockStyles(bool b);
 
-	/* these were the original functions, but shouldn't we use UT_UTF8String()?
-	 */
-	void setMetaDataProp (const UT_String & key, const UT_String & value);
-	bool getMetaDataProp (const UT_String & key, UT_String & outProp) const;
+	virtual void setMetaDataProp (const UT_String & key, const UT_UTF8String & value);
+	virtual bool getMetaDataProp (const UT_String & key, UT_UTF8String & outProp) const;
 
-	void setMetaDataProp (const UT_String & key, const UT_UTF8String & value);
-	bool getMetaDataProp (const UT_String & key, UT_UTF8String & outProp) const;
-
-	UT_StringPtrMap & getMetaData () { return m_metaDataMap ; }
+	UT_GenericStringMap<UT_UTF8String*> & getMetaData () { return m_metaDataMap ; }
 
 	// document-level property handling functions
 	const PP_AttrProp *     getAttrProp() const;
@@ -505,32 +581,76 @@ public:
 	UT_uint32 getUID(UT_UniqueId::idType t) {return m_UID.getUID(t);}
 	bool      setMinUID(UT_UniqueId::idType t, UT_uint32 i) {return m_UID.setMinId(t,i);}
 	bool      isIdUnique(UT_UniqueId::idType t, UT_uint32 i) {return m_UID.isIdUnique(t,i);}
+
+	virtual bool  areDocumentContentsEqual(const AD_Document &d, UT_uint32 &pos) const;
+	virtual bool  areDocumentFormatsEqual(const AD_Document &d, UT_uint32 &pos) const;
+	virtual bool  areDocumentStylesheetsEqual(const AD_Document &d) const;
+
+	bool      findFirstDifferenceInContent(PT_DocPosition &pos, UT_sint32 &iOffset2,
+										   const PD_Document &d) const;
+	
+	bool      findWhereSimilarityResumes(PT_DocPosition &pos, UT_sint32 &iOffset2,
+										 UT_uint32 & iKnownLength,
+										 const PD_Document &d) const;
+
+	bool      diffDocuments(const PD_Document &d, UT_Vector & vDiff) const;
+	void      diffIntoRevisions(const PD_Document &d);
+
+	virtual void   setAutoRevisioning(bool autorev);
+
+
+	UT_sint32     getNewHdrHeight(void) const
+	{ return m_iNewHdrHeight;}
+	UT_sint32     getNewFtrHeight(void) const
+	{ return m_iNewFtrHeight;}
+
+	void       setNewHdrHeight(UT_sint32 newHeight)
+	{ m_iNewHdrHeight = newHeight;}
+	void       setNewFtrHeight(UT_sint32 newHeight)
+	{ m_iNewFtrHeight = newHeight;}
+
+	bool                    purgeFmtMarks();
+
+	void                    tellPTDoNotTweakPosition(bool b);
 	
 protected:
 	~PD_Document();
 
+	virtual UT_Error		_saveAs(const char * szFilename, int ieft, const char * expProps = NULL);
+	virtual UT_Error   		_saveAs(const char * szFilename, int ieft, bool cpy, const char * expProps = NULL);
+	virtual UT_Error		_save(void);
+	
+	
 	void					_setClean(void);
 	void					_destroyDataItemData(void);
 	bool					_syncFileTypes(bool bReadSaveWriteOpen);
 
+	bool                    _acceptRejectRevision(bool bReject, UT_uint32 iStart, UT_uint32 iEnd,
+												  const PP_Revision * pRev,
+												  PP_RevisionAttr &RevAttr, pf_Frag * pf,
+												  bool & bDeleted);
+
+	virtual void            _clearUndo();
+	
+	
 public:
 	// these functions allow us to retrieve visual direction at document
 	// position pos from an associated layout. They are intended to be
 	// used by exporters into (daft) formats where such information
 	// might be required (e.g. RTF).
-	bool                    exportGetVisDirectionAtPos(PT_DocPosition pos, FriBidiCharType &type);
+	bool                    exportGetVisDirectionAtPos(PT_DocPosition pos, UT_BidiCharType &type);
 private:
 	bool                    _exportInitVisDirection(PT_DocPosition pos);
 	bool                    _exportFindVisDirectionRunAtPos(PT_DocPosition pos);
-	
+
 private:
 	bool					m_ballowListUpdates;
 	pt_PieceTable *			m_pPieceTable;
-	UT_Vector				m_vecListeners;
-	UT_Vector				m_vecLists;
+	UT_GenericVector<PL_Listener *> m_vecListeners;
+	UT_GenericVector<fl_AutoNum *> m_vecLists;
 	bool                    m_bHasListStopped;
 
-	UT_StringPtrMap		    m_hashDataItems;
+	UT_GenericStringMap<struct _dataItemPair*> m_hashDataItems;
 public:
 	IE_FileInfo				m_fileImpExpInfo;
 private:
@@ -541,18 +661,14 @@ private:
 	bool					m_bAllowInsertPointChange;
 	bool                    m_bRedrawHappenning;
 	bool                    m_bLoading;
-	bool                    m_bForcedDirty;
 	UT_Vector				m_vBookmarkNames;
 	bool                    m_bLockedStyles;
-	UT_StringPtrMap         m_metaDataMap;
-	bool                    m_bMarkRevisions;
-	UT_uint32               m_iRevisionID;
-	UT_Vector               m_vRevisions;
+	UT_GenericStringMap<UT_UTF8String*> m_metaDataMap;
 	PT_AttrPropIndex        m_indexAP;
 	bool                    m_bDontImmediatelyLayout;
 
 	// mapping UT_String=>UT_UTF8String pointer
-	UT_StringPtrMap         m_mailMergeMap;
+	UT_GenericStringMap<UT_UTF8String*>  m_mailMergeMap;
 
 	UT_UCS4Char             m_iLastDirMarker;
 
@@ -563,6 +679,9 @@ private:
 	fp_Run *                m_pVDRun;
 	PT_DocPosition          m_iVDLastPos;
 	UT_UniqueId             m_UID;
+	UT_sint32               m_iNewHdrHeight;
+	UT_sint32               m_iNewFtrHeight;
+	bool                    m_bMarginChangeOnly;
 };
 
 #endif /* PD_DOCUMENT_H */
