@@ -30,7 +30,8 @@
 
 #include "xap_App.h"
 #include "xap_QNXApp.h"
-#include "xap_QNXFrame.h"
+#include "xap_QNXFrameImpl.h"
+#include "xap_Frame.h"
 
 #include "ap_Dialog_Id.h"
 
@@ -227,10 +228,8 @@ static int s_preview_exposed(PtWidget_t * w, PhTile_t * damage)
 	//clip to our basic canvas (it's only polite).
     PtClipAdd( w, &rect );
 
-	AP_QNXDialog_Paragraph *pQNXDlg, **ppQNXDlg = NULL;
-	PtSetArg(&args[0], Pt_ARG_USER_DATA, &ppQNXDlg, 0);
-	PtGetResources(w, 1, args);
-	pQNXDlg = (ppQNXDlg) ? *ppQNXDlg : NULL;
+	AP_QNXDialog_Paragraph *pQNXDlg;
+	PtGetResource(w, Pt_ARG_POINTER, &pQNXDlg,0);
 
 	UT_ASSERT(pQNXDlg);
 	pQNXDlg->event_PreviewAreaExposed();
@@ -243,15 +242,11 @@ static int s_preview_exposed(PtWidget_t * w, PhTile_t * damage)
 
 void AP_QNXDialog_Paragraph::runModal(XAP_Frame * pFrame)
 {
-	// To center the dialog, we need the frame of its parent.
-	XAP_QNXFrame * pQNXFrame = static_cast<XAP_QNXFrame *>(pFrame);
-	UT_ASSERT(pQNXFrame);
-	
-	// Get the GtkWindow of the parent frame
-	PtWidget_t * parentWindow = pQNXFrame->getTopLevelWindow();
+	XAP_QNXFrameImpl * pQNXFrameImpl = (XAP_QNXFrameImpl*)pFrame->getFrameImpl();
+	PtWidget_t *parentWindow =	pQNXFrameImpl->getTopLevelWindow();	
 	UT_ASSERT(parentWindow);
-	PtSetParentWidget(parentWindow);
-	
+
+	PtSetParentWidget(parentWindow);	
 	m_pFrame = pFrame;
 	
 	// Build the window's widgets and arrange them
@@ -275,7 +270,10 @@ void AP_QNXDialog_Paragraph::runModal(XAP_Frame * pFrame)
 		UT_ASSERT(m_drawingareaPreview);
 
 		// make a new QNX GC
-		m_qnxGraphics = new GR_QNXGraphics(mainWindow, m_drawingareaPreview, pFrame->getApp());
+		//m_qnxGraphics = new GR_QNXGraphics(mainWindow, m_drawingareaPreview, pFrame->getApp());
+		GR_QNXAllocInfo ai(mainWindow, m_drawingareaPreview, pFrame->getApp());
+		m_qnxGraphics = (GR_QNXGraphics*) XAP_App::getApp()->newGraphics(ai);
+
 		unsigned short w, h;
 
 		// let the widget materialize
@@ -405,6 +403,7 @@ PtWidget_t * AP_QNXDialog_Paragraph::_constructWindow(void)
 {
 	// grab the string set
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
+	UT_UTF8String s;
 
 	PtWidget_t * windowParagraph;
 	PtWidget_t * listAlignment;
@@ -452,17 +451,19 @@ PtWidget_t * AP_QNXDialog_Paragraph::_constructWindow(void)
 	PtArg_t args[10];
 	int     n = 0;
 	double  inc = 0.01;
-
 	//TODO: Put a vertical group in the window first, set the offset etc.
 
 #define WIN_WIDTH  550	
 #define WIN_HEIGHT 400	
 	n = 0;
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Para_ParaTitle));
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_ParaTitle,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	PtSetArg(&args[n++], Pt_ARG_WINDOW_TITLE, unixstr, 0);
 	PtSetArg(&args[n++], Pt_ARG_WINDOW_RENDER_FLAGS, 0, ABI_MODAL_WINDOW_RENDER_FLAGS);
 	PtSetArg(&args[n++], Pt_ARG_WINDOW_MANAGED_FLAGS, 0, ABI_MODAL_WINDOW_MANAGE_FLAGS);
 	windowParagraph = PtCreateWidget(PtWindow, NULL, n, args);
+	SetupContextHelp(windowParagraph,this);
+	PtAddHotkeyHandler(windowParagraph,Pk_F1,0,Pt_HOTKEY_SYM,this,OpenHelp);
 	PtAddCallback(windowParagraph, Pt_CB_WINDOW_CLOSING, s_delete_clicked, this);
 	FREEP(unixstr);
 
@@ -647,13 +648,17 @@ PtWidget_t * AP_QNXDialog_Paragraph::_constructWindow(void)
 	PtCreateWidget( PtSeparator, NULL, sizeof(args3) / sizeof(PtArg_t), args3 );
 
 	listAlignment = PtCreateWidget( PtComboBox, NULL, sizeof(args4) / sizeof(PtArg_t), args4 );
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Para_AlignLeft)); litem[0]= unixstr;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_AlignLeft,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str()); litem[0]= unixstr;
 	PtListAddItems(listAlignment, litem, 1, 0); FREEP(unixstr);
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Para_AlignCentered)); litem[0]= unixstr;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_AlignCentered,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str()); litem[0]= unixstr;
 	PtListAddItems(listAlignment, litem, 1, 0); FREEP(unixstr);
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Para_AlignRight)); litem[0]= unixstr;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_AlignRight,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str()); litem[0]= unixstr;
 	PtListAddItems(listAlignment, litem, 1, 0); FREEP(unixstr);
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Para_AlignJustified)); litem[0]= unixstr;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_AlignJustified,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str()); litem[0]= unixstr;
 	PtListAddItems(listAlignment, litem, 1, 0); FREEP(unixstr);
 	TFSetDataInt(listAlignment, id_MENU_ALIGNMENT);
 
@@ -677,11 +682,14 @@ PtWidget_t * AP_QNXDialog_Paragraph::_constructWindow(void)
 
 	listSpecial = PtCreateWidget( PtComboBox, NULL, sizeof(args10) / sizeof(PtArg_t), args10 );
 	PtListDeleteAllItems(listSpecial);
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Para_SpecialNone)); litem[0]= unixstr;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_SpecialNone,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str()); litem[0]= unixstr;
 	PtListAddItems(listSpecial, litem, 1, 0); FREEP(unixstr);
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Para_SpecialFirstLine)); litem[0]= unixstr;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_SpecialFirstLine,s );
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str()); litem[0]= unixstr;
 	PtListAddItems(listSpecial, litem, 1, 0); FREEP(unixstr);
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Para_SpecialHanging)); litem[0]= unixstr;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_SpecialHanging,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str()); litem[0]= unixstr;
 	PtListAddItems(listSpecial, litem, 1, 0); FREEP(unixstr);
 	TFSetDataInt(listSpecial, id_MENU_SPECIAL_INDENT);
 
@@ -720,17 +728,23 @@ PtWidget_t * AP_QNXDialog_Paragraph::_constructWindow(void)
 
 	listLineSpacing = PtCreateWidget( PtComboBox, NULL, sizeof(args21) / sizeof(PtArg_t), args21 );
 	PtListDeleteAllItems(listLineSpacing);
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Para_SpacingSingle)); litem[0]= unixstr;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_SpacingSingle,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str()); litem[0]= unixstr;
 	PtListAddItems(listLineSpacing, litem, 1, 0); FREEP(unixstr);
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Para_SpacingHalf)); litem[0]= unixstr;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_SpacingHalf,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str()); litem[0]= unixstr;
 	PtListAddItems(listLineSpacing, litem, 1, 0); FREEP(unixstr);
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Para_SpacingDouble)); litem[0]= unixstr;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_SpacingDouble,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str()); litem[0]= unixstr;
 	PtListAddItems(listLineSpacing, litem, 1, 0); FREEP(unixstr);
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Para_SpacingAtLeast)); litem[0]= unixstr;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_SpacingAtLeast,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str()); litem[0]= unixstr;
 	PtListAddItems(listLineSpacing, litem, 1, 0); FREEP(unixstr);
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Para_SpacingExactly)); litem[0]= unixstr;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_SpacingExactly,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str()); litem[0]= unixstr;
 	PtListAddItems(listLineSpacing, litem, 1, 0); FREEP(unixstr);
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Para_SpacingMultiple)); litem[0]= unixstr;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_SpacingMultiple,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str()); litem[0]= unixstr;
 	PtListAddItems(listLineSpacing, litem, 1, 0); FREEP(unixstr);
 	TFSetDataInt(listLineSpacing, id_MENU_SPECIAL_SPACING);
 
@@ -752,7 +766,7 @@ PtWidget_t * AP_QNXDialog_Paragraph::_constructWindow(void)
 										 sizeof(args26) / sizeof(PtArg_t), args26);
 	n = 0;
 	void *data = (void *)this;
-	PtSetArg(&args[n++], Pt_ARG_USER_DATA, &data, sizeof(this)); 
+	PtSetArg(&args[n++], Pt_ARG_POINTER, this, 0); 
 	PtSetArg(&args[n++], Pt_ARG_RAW_DRAW_F, &s_preview_exposed, 1); 
 	PtSetResources(drawingareaPreview, n, args);
 
@@ -887,7 +901,8 @@ PtWidget_t * AP_QNXDialog_Paragraph::_constructWindow(void)
 
 	/* Now at the bottom add in some buttons ... */
 	PtSetArg(&args[n++], Pt_ARG_WIDTH, ABI_DEFAULT_BUTTON_WIDTH, 0);
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Para_ButtonTabs));
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_ButtonTabs,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, unixstr, 0);
 	PtSetArg(&args[n++], Pt_ARG_FLAGS,Pt_TRUE,Pt_GHOST|Pt_BLOCKED);
 	buttonTabs = PtCreateWidget(PtButton, hbuttongroup, n, args);
@@ -900,14 +915,16 @@ PtWidget_t * AP_QNXDialog_Paragraph::_constructWindow(void)
 
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_WIDTH, ABI_DEFAULT_BUTTON_WIDTH, 0);
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(XAP_STRING_ID_DLG_OK));
+	pSS->getValueUTF8(XAP_STRING_ID_DLG_OK,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, unixstr, 0);
 	buttonOK = PtCreateWidget(PtButton, hbuttongroup, n, args);
 	FREEP(unixstr);
 
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_WIDTH, ABI_DEFAULT_BUTTON_WIDTH, 0);
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(XAP_STRING_ID_DLG_Cancel));
+	pSS->getValueUTF8(XAP_STRING_ID_DLG_Cancel,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, unixstr, 0);
 	buttonCancel = PtCreateWidget(PtButton, hbuttongroup, n, args);
 	FREEP(unixstr);
