@@ -1,19 +1,21 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
- * Copyright (C) 1998,1999 AbiSource, Inc.
- * 
+ * Copyright (C) 1998-2000 AbiSource, Inc.
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -21,6 +23,7 @@
 #include <stdio.h>
 #include <string.h>
 #include "ut_assert.h"
+#include "ut_exception.h"
 #include "ut_string.h"
 #include "ut_growbuf.h"
 #include "ut_debugmsg.h"
@@ -45,67 +48,43 @@
 #include "pp_Property.h"
 
 AP_Dialog_Paragraph::AP_Dialog_Paragraph(XAP_DialogFactory* pDlgFactory, XAP_Dialog_Id id)
-	: XAP_Dialog_NonPersistent(pDlgFactory,id)
+	: XAP_Dialog_NonPersistent(pDlgFactory,id, "interface/dialogparagraph")
 {
 	m_answer = a_OK;
 	m_paragraphPreview = NULL;
 	m_pFrame = NULL;
-	
+
 	// determine unit system to use in this dialog
 	const XML_Char * szRulerUnits;
-	UT_ASSERT(m_pApp);
-	
+	UT_return_if_fail (m_pApp);
+
 	XAP_Prefs* pPrefs = m_pApp->getPrefs();
-	UT_ASSERT(pPrefs);
+	UT_return_if_fail (pPrefs);
 
 	const bool bHasRulerUnits =
 		pPrefs->getPrefsValue((XML_Char*)AP_PREF_KEY_RulerUnits, &szRulerUnits);
 
 	m_dim = bHasRulerUnits ? UT_determineDimension(szRulerUnits) : DIM_IN;
 
-	// terminate the static buffers for cleanliness
-	m_bufLeftIndent[0] = 0;
-	m_bufRightIndent[0] = 0;
-	m_bufSpecialIndent[0] = 0;
-	m_bufBeforeSpacing[0] = 0;
-	m_bufAfterSpacing[0] = 0;
-	m_bufSpecialSpacing[0] = 0;
-
 	m_pageLeftMargin = NULL;
 	m_pageRightMargin = NULL;
 
-	// initialize vector of control/value/changed items
-	struct id_val_pair
-	{
-		tControl	index;
-		void*		pValue;
-	};
-	const id_val_pair rgPairs[] =
-	{
-		{id_MENU_ALIGNMENT,			(void*)align_LEFT},
-		{id_SPIN_LEFT_INDENT,		&m_bufLeftIndent},
-		{id_SPIN_RIGHT_INDENT,		&m_bufRightIndent},
-		{id_MENU_SPECIAL_INDENT,	(void*)indent_NONE},
-		{id_SPIN_SPECIAL_INDENT,	&m_bufSpecialIndent},
-		{id_SPIN_BEFORE_SPACING,	&m_bufBeforeSpacing},
-		{id_SPIN_AFTER_SPACING,		&m_bufAfterSpacing},
-		{id_MENU_SPECIAL_SPACING,	(void*)spacing_SINGLE},
-		{id_SPIN_SPECIAL_SPACING,	&m_bufSpecialSpacing},
-		{id_CHECK_WIDOW_ORPHAN,		(void*)check_INDETERMINATE},
-		{id_CHECK_KEEP_LINES,		(void*)check_INDETERMINATE},
-		{id_CHECK_PAGE_BREAK,		(void*)check_INDETERMINATE},
-		{id_CHECK_SUPPRESS,			(void*)check_INDETERMINATE},
-		{id_CHECK_NO_HYPHENATE,		(void*)check_INDETERMINATE},
-		{id_CHECK_KEEP_NEXT,		(void*)check_INDETERMINATE}
-#ifdef BIDI_ENABLED		
-		,{id_CHECK_DOMDIRECTION,	(void*)check_TRUE}
-#endif		
-	};
-
-	for (unsigned int i = 0; i < NrElements(rgPairs); ++i)
-	{
-		_addPropertyItem(rgPairs[i].index, rgPairs[i].pValue);
-	}
+	_addPropertyItem (id_MENU_ALIGNMENT,		sControlData(align_LEFT));
+	_addPropertyItem (id_SPIN_LEFT_INDENT,		sControlData());
+	_addPropertyItem (id_SPIN_RIGHT_INDENT,		sControlData());
+	_addPropertyItem (id_MENU_SPECIAL_INDENT,	sControlData(indent_NONE));
+	_addPropertyItem (id_SPIN_SPECIAL_INDENT,	sControlData());
+	_addPropertyItem (id_SPIN_BEFORE_SPACING,	sControlData());
+	_addPropertyItem (id_SPIN_AFTER_SPACING,	sControlData());
+	_addPropertyItem (id_MENU_SPECIAL_SPACING,	sControlData(spacing_SINGLE));
+	_addPropertyItem (id_SPIN_SPECIAL_SPACING,	sControlData());
+	_addPropertyItem (id_CHECK_WIDOW_ORPHAN,	sControlData(check_INDETERMINATE));
+	_addPropertyItem (id_CHECK_KEEP_LINES,		sControlData(check_INDETERMINATE));
+	_addPropertyItem (id_CHECK_PAGE_BREAK,		sControlData(check_INDETERMINATE));
+	_addPropertyItem (id_CHECK_SUPPRESS,		sControlData(check_INDETERMINATE));
+	_addPropertyItem (id_CHECK_NO_HYPHENATE,	sControlData(check_INDETERMINATE));
+	_addPropertyItem (id_CHECK_KEEP_NEXT,		sControlData(check_INDETERMINATE));
+	_addPropertyItem (id_CHECK_DOMDIRECTION,	sControlData(check_TRUE));
 }
 
 AP_Dialog_Paragraph::~AP_Dialog_Paragraph(void)
@@ -115,26 +94,22 @@ AP_Dialog_Paragraph::~AP_Dialog_Paragraph(void)
 
 	DELETEP(m_paragraphPreview);
 
-	// This macro will remove all the sControlData items in
-	// the vector, but will not free the sControlData.pValue items
-	// (which is desirable, they will be either ints or static
-	// buffers).
 	UT_VECTOR_PURGEALL(sControlData *, m_vecProperties);
 }
 
 bool AP_Dialog_Paragraph::setDialogData(const XML_Char ** pProps)
 {
-	UT_ASSERT(pProps);
+	UT_return_val_if_fail (pProps, false);
 
 	// NOTICE : When setting values, this function always calls
 	// NOTICE : _set[thing]ItemValue() with the bToggleDirty flag
 	// NOTICE : set to false, because these are the "un-dirty"
 	// NOTICE : values.
-	
+
 	if (pProps[0])
 	{
 		const XML_Char * sz;
-		
+
 		sz = UT_getAttribute("text-align", pProps);
 		if (sz)
 		{
@@ -149,12 +124,12 @@ bool AP_Dialog_Paragraph::setDialogData(const XML_Char ** pProps)
 			else if (UT_XML_strcmp(sz, "left") == 0)
 				t = align_LEFT;
 			else
-				UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+				UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 
-			_setMenuItemValue(id_MENU_ALIGNMENT, t, op_INIT);			
+			_setMenuItemValue(id_MENU_ALIGNMENT, t, op_INIT);
 		}
-#ifdef BIDI_ENABLED		
-		sz = UT_getAttribute("dom-dir", pProps);						
+
+		sz = UT_getAttribute("dom-dir", pProps);
 		if (sz)
 		{
 			tCheckState t = check_FALSE;
@@ -164,11 +139,11 @@ bool AP_Dialog_Paragraph::setDialogData(const XML_Char ** pProps)
 			else if (UT_XML_strcmp(sz, "rtl") == 0)
 				t = check_TRUE;
 			else
-				UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+				UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 
-			_setCheckItemValue(id_CHECK_DOMDIRECTION, t, op_INIT);			
+			_setCheckItemValue(id_CHECK_DOMDIRECTION, t, op_INIT);
 		}
-#endif		
+
 		sz = UT_getAttribute("margin-left", pProps);
 		if (sz)
 			_setSpinItemValue(id_SPIN_LEFT_INDENT, sz, op_INIT);
@@ -210,7 +185,7 @@ bool AP_Dialog_Paragraph::setDialogData(const XML_Char ** pProps)
 
 			if (sz[0] == '-')
 				newSz++;
-			
+
 			_setSpinItemValue(id_SPIN_SPECIAL_INDENT, newSz, op_INIT);
 		}
 
@@ -227,8 +202,8 @@ bool AP_Dialog_Paragraph::setDialogData(const XML_Char ** pProps)
 
 					// need to strip off that plus
 					int posPlus = pPlusFound - (char*) sz;
-					UT_ASSERT(posPlus>=0);
-					UT_ASSERT(posPlus<100);
+					UT_return_val_if_fail (posPlus>=0, false);
+					UT_return_val_if_fail (posPlus<100, false);
 
 					char pTmp[100];
 					strcpy(pTmp, sz);
@@ -259,11 +234,11 @@ bool AP_Dialog_Paragraph::setDialogData(const XML_Char ** pProps)
 		sz = UT_getAttribute("margin-top", pProps);
 		if (sz)
 			_setSpinItemValue(id_SPIN_BEFORE_SPACING, sz, op_INIT);
-			
+
 		sz = UT_getAttribute("margin-bottom", pProps);
 		if (sz)
 			_setSpinItemValue(id_SPIN_AFTER_SPACING, sz, op_INIT);
-		
+
 		{
 			// NOTE : "orphans" and "widows" hold a number specifying the number
 			// NOTE : of lines to consider an orphaned or widowed piece of text.
@@ -277,13 +252,13 @@ bool AP_Dialog_Paragraph::setDialogData(const XML_Char ** pProps)
 			bool bNoWidows = false;
 
 			double orphans = 0, widows = 0;
-			
+
 			sz = UT_getAttribute("orphans", pProps);
 			if (sz)
 				orphans = UT_convertDimensionless(sz);
 			else
 				bNoOrphans = true;
-			
+
 			sz = UT_getAttribute("widows", pProps);
 			if (sz)
 				widows = UT_convertDimensionless(sz);
@@ -331,7 +306,7 @@ bool AP_Dialog_Paragraph::setDialogData(const XML_Char ** pProps)
 		  	UT_XML_cloneString(m_pageLeftMargin,
 					   PP_lookupProperty("page-margin-left")->getInitial());
 		}
-			
+
 		sz = UT_getAttribute("page-margin-right", pProps);
 		if (sz)
 		{
@@ -342,7 +317,7 @@ bool AP_Dialog_Paragraph::setDialogData(const XML_Char ** pProps)
 		  	UT_XML_cloneString(m_pageRightMargin,
 					   PP_lookupProperty("page-margin-right")->getInitial());
 		}
-			
+
 		// TODO : add these to PP_Property (pp_Property.cpp) !!!
 		// TODO : and to FV_View::getBlockFormat (or else they won't come in)
 		/*
@@ -378,7 +353,7 @@ bool AP_Dialog_Paragraph::getDialogData(const XML_Char **& pProps)
 	};
 
 	propPair * p;
-		
+
 	if (_wasChanged(id_MENU_ALIGNMENT))
 	{
 		ALLOC_PROP_PAIR(p);
@@ -399,12 +374,12 @@ bool AP_Dialog_Paragraph::getDialogData(const XML_Char **& pProps)
 			UT_XML_cloneString(p->val, "justify");
 			break;
 		default:
-			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		}
 		v.addItem(p);
 	}
 
-#ifdef BIDI_ENABLED
+
 	if (_wasChanged(id_CHECK_DOMDIRECTION))
 	{
 		ALLOC_PROP_PAIR(p);
@@ -418,8 +393,8 @@ bool AP_Dialog_Paragraph::getDialogData(const XML_Char **& pProps)
 		v.addItem(p);
 
 	}
-#endif
-	
+
+
 	if (_wasChanged(id_SPIN_LEFT_INDENT))
 	{
 		ALLOC_PROP_PAIR(p);
@@ -428,7 +403,7 @@ bool AP_Dialog_Paragraph::getDialogData(const XML_Char **& pProps)
 
 		v.addItem(p);
 	}
-	
+
 	if (_wasChanged(id_SPIN_RIGHT_INDENT))
 	{
 		ALLOC_PROP_PAIR(p);
@@ -482,7 +457,7 @@ bool AP_Dialog_Paragraph::getDialogData(const XML_Char **& pProps)
 
 		v.addItem(p);
 	}
-	
+
 	if (_wasChanged(id_SPIN_AFTER_SPACING))
 	{
 		ALLOC_PROP_PAIR(p);
@@ -491,7 +466,7 @@ bool AP_Dialog_Paragraph::getDialogData(const XML_Char **& pProps)
 
 		v.addItem(p);
 	}
-	
+
 	// TODO : The logic here might not be bulletproof.  If the user triggers
 	// TODO : a change in the TYPE of special indent (single, double, etc.)
 	// TODO : we will always save what's in the box as a property.
@@ -502,7 +477,7 @@ bool AP_Dialog_Paragraph::getDialogData(const XML_Char **& pProps)
 	{
 		ALLOC_PROP_PAIR(p);
 		UT_XML_cloneString(p->prop, "line-height");
-		
+
 		// normal spacings (single, 1.5, double) are just simple numbers.
 		// "at least" needs a "+" at the end of the number (no units).
 		// "exactly" simply has units.
@@ -511,7 +486,7 @@ bool AP_Dialog_Paragraph::getDialogData(const XML_Char **& pProps)
 		XML_Char * pTmp = NULL;
 		const XML_Char * pString = _getSpinItemValue(id_SPIN_SPECIAL_SPACING);
 		UT_uint32 nSize = 0;
-		
+
 		switch(_getMenuItemValue(id_MENU_SPECIAL_SPACING))
 		{
 		case spacing_SINGLE:
@@ -526,7 +501,7 @@ bool AP_Dialog_Paragraph::getDialogData(const XML_Char **& pProps)
 		case spacing_ATLEAST:
 			nSize = UT_XML_strlen(pString);
 			pTmp = (XML_Char *) UT_calloc(nSize + 2, sizeof(XML_Char));
-			UT_ASSERT(pTmp);
+			UT_return_val_if_fail (pTmp, false);
 
 			UT_XML_strncpy(pTmp, nSize, pString);
 			// stick a '+' at the end
@@ -543,7 +518,7 @@ bool AP_Dialog_Paragraph::getDialogData(const XML_Char **& pProps)
 			UT_XML_cloneString(p->val, pString);
 			break;
 		default:
-			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		}
 		v.addItem(p);
 	}
@@ -559,11 +534,11 @@ bool AP_Dialog_Paragraph::getDialogData(const XML_Char **& pProps)
 		// TODO : fix this!  we stomp on both properties (setting them
 		// TODO : to "2"s or "0"s) if the one check box was ever
 		// TODO : changed
-		
+
 		{
 			ALLOC_PROP_PAIR(p);
 			UT_XML_cloneString(p->prop, "orphans");
-		
+
 			if (_getCheckItemValue(id_CHECK_WIDOW_ORPHAN) == check_TRUE)
 				UT_XML_cloneString(p->val, "2");
 			else
@@ -571,11 +546,11 @@ bool AP_Dialog_Paragraph::getDialogData(const XML_Char **& pProps)
 
 			v.addItem(p);
 		}
-		
+
 		{
 			ALLOC_PROP_PAIR(p);
 			UT_XML_cloneString(p->prop, "widows");
-		
+
 			if (_getCheckItemValue(id_CHECK_WIDOW_ORPHAN) == check_TRUE)
 				UT_XML_cloneString(p->val, "2");
 			else
@@ -584,7 +559,7 @@ bool AP_Dialog_Paragraph::getDialogData(const XML_Char **& pProps)
 			v.addItem(p);
 		}
 	}
-	
+
 	if (_wasChanged(id_CHECK_KEEP_LINES))
 	{
 		ALLOC_PROP_PAIR(p);
@@ -597,7 +572,7 @@ bool AP_Dialog_Paragraph::getDialogData(const XML_Char **& pProps)
 
 		v.addItem(p);
 	}
-	
+
 	if (_wasChanged(id_CHECK_KEEP_NEXT))
 	{
 		ALLOC_PROP_PAIR(p);
@@ -646,7 +621,7 @@ bool AP_Dialog_Paragraph::getDialogData(const XML_Char **& pProps)
 	// by the pointers we just copied into memory typed (XML_Char **)
 
 	pProps = newprops;
-	
+
 	return true;
 }
 
@@ -666,24 +641,24 @@ void AP_Dialog_Paragraph::_createPreviewFromGC(GR_Graphics * gc,
 											   UT_uint32 width,
 											   UT_uint32 height)
 {
-	UT_ASSERT(gc);
+	UT_return_if_fail (gc);
 
 	// free any attached preview
 	DELETEP(m_paragraphPreview);
 
 	// platform's runModal should have set this
-	UT_ASSERT(m_pFrame);
+	UT_return_if_fail (m_pFrame);
 
 	AV_View * baseview = m_pFrame->getCurrentView();
-	UT_ASSERT(baseview);
+	UT_return_if_fail (baseview);
 
 	FV_View * view = static_cast<FV_View *> (baseview);
 
 	FL_DocLayout * dl = view->getLayout();
-	UT_ASSERT(dl);
+	UT_return_if_fail (dl);
 
 	fl_BlockLayout * bl = dl->findBlockAtPosition((PT_DocPosition) view->getPoint());
-	UT_ASSERT(bl);
+	UT_return_if_fail (bl);
 
 	UT_GrowBuf gb;
 	bool hadMem = bl->getBlockBuf(&gb);
@@ -692,22 +667,22 @@ void AP_Dialog_Paragraph::_createPreviewFromGC(GR_Graphics * gc,
 	if (hadMem && gb.getLength() > 0)
 	{
 		gb.truncate(NUM_CHARS_FOR_SAMPLE);
-		UT_UCS_cloneString(&tmp, (UT_UCSChar *) gb.getPointer(0));
+		UT_UCS4_cloneString(&tmp, (UT_UCSChar *) gb.getPointer(0));
 	}
 	else
 	{
 		const XAP_StringSet * pSS = m_pApp->getStringSet();
-	
+
 		// if the paragraph was empty, use our sample
-		UT_UCS_cloneString_char(&tmp, pSS->getValue(AP_STRING_ID_DLG_Para_PreviewSampleFallback));
+		UT_UCS4_cloneString_char(&tmp, pSS->getValue(AP_STRING_ID_DLG_Para_PreviewSampleFallback));
 	}
 
 	m_paragraphPreview = new AP_Preview_Paragraph(gc, tmp, this);
 
 	FREEP(tmp);
-	
-	UT_ASSERT(m_paragraphPreview);
-	
+
+	UT_return_if_fail (m_paragraphPreview);
+
 	m_paragraphPreview->setWindowSize(width, height);
 
 	// TODO : any setup of the GC for drawing
@@ -717,16 +692,15 @@ void AP_Dialog_Paragraph::_createPreviewFromGC(GR_Graphics * gc,
 void AP_Dialog_Paragraph::_setMenuItemValue(tControl item, UT_sint32 value,
 											tOperation op /* = op_UICHANGE */)
 {
-	UT_ASSERT((UT_uint32) item <= m_vecProperties.getItemCount());
+	UT_return_if_fail ((UT_uint32) item <= m_vecProperties.getItemCount());
 
-	sControlData * pItem = (sControlData *) m_vecProperties.getNthItem((UT_uint32) item);
-	UT_ASSERT(pItem);
+	sControlData * pItem = _getPropertyItem (item);
+	UT_return_if_fail (pItem);
 
-	// menu items have integers as data, so store it in a pointer
-	pItem->pData = (void *) value;
+	pItem->setData (value);
 
 	if ((op == op_UICHANGE) || (op == op_SYNC))
-		pItem->bChanged = true;
+		pItem->changed (true);
 
 	// for UI-driven changes, may need to sync other controls
 	if (op == op_UICHANGE)
@@ -734,29 +708,30 @@ void AP_Dialog_Paragraph::_setMenuItemValue(tControl item, UT_sint32 value,
 }
 
 
-UT_uint32 AP_Dialog_Paragraph::_getMenuItemValue(tControl item)
+UT_sint32 AP_Dialog_Paragraph::_getMenuItemValue(tControl item)
 {
-	UT_ASSERT((UT_uint32) item <= m_vecProperties.getItemCount());
+	UT_return_val_if_fail ((UT_uint32) item <= m_vecProperties.getItemCount(), 0);
 
-	sControlData * pItem = (sControlData *) m_vecProperties.getNthItem((UT_uint32) item);
-	UT_ASSERT(pItem);
+	sControlData * pItem = _getPropertyItem (item);
+	UT_return_val_if_fail (pItem, 0);
 
-	return (UT_uint32) pItem->pData;
+	UT_sint32 value = 0;
+	pItem->getData (value);
+	return value;
 }
-		
+
 void AP_Dialog_Paragraph::_setCheckItemValue(tControl item, tCheckState value,
 											tOperation op /* = op_UICHANGE */)
 {
-	UT_ASSERT((UT_uint32) item <= m_vecProperties.getItemCount());
+	UT_return_if_fail ((UT_uint32) item <= m_vecProperties.getItemCount());
 
-	sControlData * pItem = (sControlData *) m_vecProperties.getNthItem((UT_uint32) item);
-	UT_ASSERT(pItem);
+	sControlData * pItem = _getPropertyItem (item);
+	UT_return_if_fail (pItem);
 
-	// check buttons have integers as data, so store it in a pointer
-	pItem->pData = (void *) value;
+	pItem->setData (value);
 
 	if ((op == op_UICHANGE) || (op == op_SYNC))
-		pItem->bChanged = true;
+		pItem->changed (true);
 
 	// for UI-driven changes, may need to sync other controls
 	if (op == op_UICHANGE)
@@ -765,14 +740,15 @@ void AP_Dialog_Paragraph::_setCheckItemValue(tControl item, tCheckState value,
 
 AP_Dialog_Paragraph::tCheckState AP_Dialog_Paragraph::_getCheckItemValue(tControl item)
 {
-	UT_ASSERT((UT_uint32) item <= m_vecProperties.getItemCount());
+	UT_return_val_if_fail ((UT_uint32) item <= m_vecProperties.getItemCount(), check_INDETERMINATE);
 
-	sControlData * pItem = (sControlData *) m_vecProperties.getNthItem((UT_uint32) item);
-	UT_ASSERT(pItem);
+	sControlData * pItem = _getPropertyItem (item);
+	UT_return_val_if_fail (pItem, check_INDETERMINATE);
 
-	// stupid casting tricks to keep the compiler happy
-	UT_uint32 tmp = (UT_uint32) pItem->pData;
-	return (tCheckState) tmp;
+	tCheckState value = check_INDETERMINATE;
+	if (pItem)
+		pItem->getData (value);
+	return value;
 }
 
 const XML_Char * AP_Dialog_Paragraph::_makeAbsolute(const XML_Char * value)
@@ -797,56 +773,51 @@ const XML_Char * AP_Dialog_Paragraph::_makeAbsolute(const XML_Char * value)
 void AP_Dialog_Paragraph::_setSpinItemValue(tControl item, const XML_Char * value,
 											tOperation op /* = op_UICHANGE */)
 {
-	UT_ASSERT((UT_uint32) item <= m_vecProperties.getItemCount());
+	UT_return_if_fail ((UT_uint32) item <= m_vecProperties.getItemCount() && value);
 
-	sControlData * pItem = (sControlData *) m_vecProperties.getNthItem((UT_uint32) item);
-	UT_ASSERT(pItem);
-	
-	// spin items have pointers to static XML_Char buffers as data, so we
-	// don't free them, but extract a pointer and copy the string contents
+	sControlData * pItem = _getPropertyItem (item);
+	UT_return_if_fail (pItem);
 
-	UT_ASSERT(pItem->pData && value);
-	
 	// some spinbuttons have special data requirements
 	switch(item)
 	{
 	case id_SPIN_LEFT_INDENT:
 	case id_SPIN_RIGHT_INDENT:
-		{
-		UT_XML_strncpy((XML_Char *) pItem->pData, SPIN_BUF_TEXT_SIZE, UT_reformatDimensionString(m_dim, value));
-		}
-		break;
-
 	case id_SPIN_SPECIAL_INDENT:
-
-		UT_XML_strncpy((XML_Char *) pItem->pData, SPIN_BUF_TEXT_SIZE, UT_reformatDimensionString(m_dim, value));			
+		pItem->setData (reinterpret_cast<const XML_Char *>(UT_reformatDimensionString (m_dim, value)));
 		break;
-		
+
 	case id_SPIN_BEFORE_SPACING:
 	case id_SPIN_AFTER_SPACING:
-		// NOTE : line spacing can't be negative, so make absolut
-
-		UT_XML_strncpy((XML_Char *) pItem->pData, SPIN_BUF_TEXT_SIZE, UT_reformatDimensionString(DIM_PT, _makeAbsolute(value)));
+		{
+			/* NOTE : line spacing can't be negative, so take absolute value:
+			 */
+			const char * abs_value = UT_reformatDimensionString (DIM_PT, _makeAbsolute (value));
+			pItem->setData (reinterpret_cast<const XML_Char *>(abs_value));
+		}
 		break;
-		
+
 	case id_SPIN_SPECIAL_SPACING:
-		if(_getMenuItemValue(id_MENU_SPECIAL_SPACING) == spacing_MULTIPLE)
-		{
-			UT_XML_strncpy((XML_Char *) pItem->pData, SPIN_BUF_TEXT_SIZE, UT_reformatDimensionString(DIM_none, _makeAbsolute(value),".2"));
-		}
+		if (_getMenuItemValue (id_MENU_SPECIAL_SPACING) == spacing_MULTIPLE)
+			{
+				const char * abs_value = UT_reformatDimensionString (DIM_none, _makeAbsolute (value), ".2");
+				pItem->setData (reinterpret_cast<const XML_Char *>(abs_value));
+			}
 		else
-		{
-			UT_XML_strncpy((XML_Char *) pItem->pData, SPIN_BUF_TEXT_SIZE, UT_reformatDimensionString(DIM_PT, _makeAbsolute(value)));
-		}
+			{
+				const char * abs_value = UT_reformatDimensionString (DIM_PT, _makeAbsolute (value));
+				pItem->setData (reinterpret_cast<const XML_Char *>(abs_value));
+			}
 		break;
 
 	default:
-		// all others get a simple string copy to the static member
-		UT_XML_strncpy((XML_Char *) pItem->pData, SPIN_BUF_TEXT_SIZE, value);
+		/* all others get a simple string copy to the static member
+		 */
+		pItem->setData (value);
 	}
 
 	if ((op == op_UICHANGE) || (op == op_SYNC))
-		pItem->bChanged = true;
+		pItem->changed (true);
 
 	// for UI-driven changes, may need to sync other controls
 	if (op == op_UICHANGE)
@@ -855,12 +826,15 @@ void AP_Dialog_Paragraph::_setSpinItemValue(tControl item, const XML_Char * valu
 
 const XML_Char * AP_Dialog_Paragraph::_getSpinItemValue(tControl item)
 {
-	UT_ASSERT((UT_uint32) item <= m_vecProperties.getItemCount());
+	UT_return_val_if_fail ((UT_uint32) item <= m_vecProperties.getItemCount(), NULL);
 
-	sControlData * pItem = (sControlData *) m_vecProperties.getNthItem((UT_uint32) item);
-	UT_ASSERT(pItem && pItem->pData);
-	
-	return (const XML_Char *) pItem->pData;
+	sControlData * pItem = _getPropertyItem (item);
+	UT_return_val_if_fail (pItem, NULL);
+
+	const XML_Char * value = NULL;
+	pItem->getData (value);
+	UT_ASSERT_HARMLESS(value);
+	return value;
 }
 
 
@@ -878,7 +852,7 @@ const XML_Char * AP_Dialog_Paragraph::_getSpinItemValue(tControl item)
 
 void AP_Dialog_Paragraph::_doSpin(tControl edit, UT_sint32 amt)
 {
-	UT_ASSERT(amt); // zero makes no sense
+	UT_ASSERT_HARMLESS(amt); // zero makes no sense
 
 	// get current value from member
 	const XML_Char* szOld = _getSpinItemValue(edit);
@@ -906,7 +880,7 @@ void AP_Dialog_Paragraph::_doSpin(tControl edit, UT_sint32 amt)
 		case DIM_PI:	dSpinUnit = SPIN_INCR_PI;	break;
 		case DIM_PT:	dSpinUnit = SPIN_INCR_PT;	break;
 		default:
-			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 			break;
 		}
 		break;
@@ -944,13 +918,13 @@ void AP_Dialog_Paragraph::_doSpin(tControl edit, UT_sint32 amt)
 			break;
 
 		default:
-			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 			break;
 		}
 		break;
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -995,8 +969,8 @@ void AP_Dialog_Paragraph::_doSpin(tControl edit, UT_sint32 amt)
 // must be to call the XP (parent) implementation
 //
 // also, the platform code has to call _syncControls() once with
-// bAll set to true.  this should happen *after* all of the 
-// member variables have been copied to the screen for the first 
+// bAll set to true.  this should happen *after* all of the
+// member variables have been copied to the screen for the first
 // time, but before the dialog is displayed.
 
 void AP_Dialog_Paragraph::_syncControls(tControl changed, bool bAll /* = false */)
@@ -1086,7 +1060,7 @@ void AP_Dialog_Paragraph::_syncControls(tControl changed, bool bAll /* = false *
 				break;
 
 			default:
-				UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+				UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 				break;
 			}
 
@@ -1094,9 +1068,9 @@ void AP_Dialog_Paragraph::_syncControls(tControl changed, bool bAll /* = false *
 			{
 				if (m_dim != DIM_IN)
 					dDefault = UT_convertInchesToDimension(dDefault, m_dim);
-	
+
 				const XML_Char* szNew = UT_convertInchesToDimensionString(m_dim, dDefault, ".1");
-	
+
 				_setSpinItemValue(id_SPIN_SPECIAL_INDENT, szNew, op_SYNC);
 			}
 		}
@@ -1107,7 +1081,7 @@ void AP_Dialog_Paragraph::_syncControls(tControl changed, bool bAll /* = false *
 			case indent_NONE:
 				_setMenuItemValue(id_MENU_SPECIAL_INDENT, indent_FIRSTLINE, op_SYNC);
 				break;
-	
+
 			default:
 				break;
 			}
@@ -1200,7 +1174,7 @@ void AP_Dialog_Paragraph::_syncControls(tControl changed, bool bAll /* = false *
 			break;
 
 		default:
-			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 			break;
 		}
 	}
@@ -1218,28 +1192,132 @@ void AP_Dialog_Paragraph::_syncControls(tControl changed, bool bAll /* = false *
 									_getSpinItemValue(id_SPIN_AFTER_SPACING),
 									_getSpinItemValue(id_SPIN_SPECIAL_SPACING),
 									(AP_Dialog_Paragraph::tSpacingState) _getMenuItemValue(id_MENU_SPECIAL_SPACING));
-	
+
 	m_paragraphPreview->draw();
 }
 
 bool AP_Dialog_Paragraph::_wasChanged(tControl item)
 {
-	UT_ASSERT((UT_uint32) item <= m_vecProperties.getItemCount());
+	UT_return_val_if_fail ((UT_uint32) item <= m_vecProperties.getItemCount(), false);
 
-	sControlData * pItem = (sControlData *) m_vecProperties.getNthItem((UT_uint32) item);
-	UT_ASSERT(pItem);
-	
-	return pItem->bChanged;
+	sControlData * pItem = _getPropertyItem (item);
+	UT_return_val_if_fail (pItem, false);
+
+	return pItem->changed ();
 }
 
-void AP_Dialog_Paragraph::_addPropertyItem(UT_uint32 index, void* pValue)
+void AP_Dialog_Paragraph::_addPropertyItem (tControl index, const sControlData & control_data)
 {
-	typedef AP_Dialog_Paragraph::sControlData sControlData;
-	sControlData* pItem = new sControlData;
-    UT_ASSERT(pItem);
-    pItem->bChanged = false;
-    pItem->pData = pValue;
-    void* pTmp;
-    m_vecProperties.setNthItem(index, pItem, &pTmp);
+	sControlData * pDataCopy = 0;
+
+	UT_TRY
+		{
+			pDataCopy = new sControlData(control_data);
+		}
+	UT_CATCH(...)
+		{
+			pDataCopy = 0;
+		}
+	UT_return_if_fail (pDataCopy);
+
+	m_vecProperties.setNthItem (static_cast<UT_uint32>(index), pDataCopy, 0);
 }
 
+AP_Dialog_Paragraph::sControlData::sControlData (UT_sint32 data) :
+	m_siData(data),
+	m_csData(check_INDETERMINATE),
+	m_szData(0),
+	m_bChanged(false)
+{
+	// 
+}
+
+AP_Dialog_Paragraph::sControlData::sControlData (tCheckState data) :
+	m_siData(0),
+	m_csData(data),
+	m_szData(0),
+	m_bChanged(false)
+{
+	// 
+}
+
+/* default is empty string
+ */
+AP_Dialog_Paragraph::sControlData::sControlData (XML_Char * data) :
+	m_siData(0),
+	m_csData(check_INDETERMINATE),
+	m_szData(new XML_Char[SPIN_BUF_TEXT_SIZE]),
+	m_bChanged(false)
+{
+	m_szData[SPIN_BUF_TEXT_SIZE-1] = 0;
+	setData (data);
+}
+
+AP_Dialog_Paragraph::sControlData::sControlData (const sControlData & rhs) :
+	m_siData(rhs.m_siData),
+	m_csData(rhs.m_csData),
+	m_szData(rhs.m_szData ? new XML_Char[SPIN_BUF_TEXT_SIZE] : 0),
+	m_bChanged(false)
+{
+	if (m_szData)
+		memcpy (m_szData, rhs.m_szData, SPIN_BUF_TEXT_SIZE * sizeof (XML_Char));
+}
+
+AP_Dialog_Paragraph::sControlData::~sControlData ()
+{
+	if (m_szData) delete [] m_szData;
+}
+
+AP_Dialog_Paragraph::sControlData & AP_Dialog_Paragraph::sControlData::operator= (const sControlData & rhs)
+{
+	m_siData = rhs.m_siData;
+	m_csData = rhs.m_csData;
+
+	if (rhs.m_szData)
+		{
+			if (!m_szData)
+				{
+					UT_TRY
+						{
+							m_szData = new XML_Char[SPIN_BUF_TEXT_SIZE];
+						}
+					UT_CATCH(...)
+						{
+							m_szData = 0;
+						}
+				}
+			UT_return_val_if_fail (m_szData, *this);
+			memcpy (m_szData, rhs.m_szData, SPIN_BUF_TEXT_SIZE * sizeof (XML_Char));
+		}
+	else if (m_szData)
+		{
+			m_szData[0] = 0;
+		}
+	m_bChanged = false;
+
+	return *this;
+}
+
+bool AP_Dialog_Paragraph::sControlData::setData (const XML_Char * data)
+{
+	if (!m_szData)
+		{
+			UT_TRY
+				{
+					m_szData = new XML_Char[SPIN_BUF_TEXT_SIZE];
+				}
+			UT_CATCH(...)
+				{
+					m_szData = 0;
+				}
+			UT_return_val_if_fail (m_szData, false);
+
+			m_szData[SPIN_BUF_TEXT_SIZE-1] = 0;
+		}
+	if (data)
+		UT_XML_strncpy (m_szData, SPIN_BUF_TEXT_SIZE - 1, data);
+	else
+		m_szData[0] = 0;
+
+	return true;
+}
