@@ -16,7 +16,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-abi_version=2.0.10
+rm -rf autom4te.cache
+
+abi_version=2.2.1
 
 # abi_plugins is the list of subdirectories that are plugins
 # The definition of a plugin is a directory containing a file called "plugin.m4".
@@ -499,65 +501,6 @@ if test -f plugin.status; then
 fi
 EOF
 
-# Create configure.in
-# 
-rm -f configure.in
-cat > configure.in << EOF
-AC_INIT(ac-helpers/abi-plugin-init.m4)
-
-dnl First of all AbiWord plugin macros:
-dnl 
-ABI_PLUGIN_INIT($abi_version)
-ABI_PLUGIN_CONFIG
-
-# Checks for programs.
-AC_PROG_CC
-AC_PROG_CPP
-AC_PROG_CXX
-AC_PROG_AWK
-AC_PROG_INSTALL
-AC_PROG_LN_S
-AC_PROG_MAKE_SET
-
-AM_INIT_AUTOMAKE(abiword, $abi_version)
-
-AC_DISABLE_STATIC
-AC_PROG_LIBTOOL
-
-AC_C_CONST
-AC_C_INLINE
-
-dnl Configure plugin sources
-dnl 
-dnl AC_CONFIG_SUBDIRS($abi_configs)
-
-abi_plugin_extra=
-abi_plugin_opt=yes
-AC_ARG_ENABLE(all,[  --disable-all           disable all plugins by default],[
-	if test "x\$enableval" = "xno"; then
-		abi_plugin_opt=no
-	fi
-])
-
-EOF
-($CONFIG_SHELL $abi_topdir/nextgen.sh --config) >> configure.in
-cat >> configure.in << EOF
-AC_OUTPUT([
-$abi_makefiles
-EOF
-($CONFIG_SHELL $abi_topdir/nextgen.sh --query --list-files) >> configure.in
-cat >> configure.in << EOF
-])
-
-if test -f plugin.status; then
-	echo ""
-	echo "Plugin reports:"
-	echo ""
-	cat plugin.status
-	echo ""
-fi
-EOF
-
 # And, finally, call in the big boys...
 # 
 rm -f autogen.out
@@ -633,3 +576,5 @@ echo "Running autoconf"
   echo ">autoconf failed! Sorry - I'm unable to continue."
   exit 1
 }
+
+rm -rf autom4ate.cache
